[![Build Status][badge-travis-image]][badge-travis-url]

# kong-plugin-enterprise-canary

Kong plugin for Canary release of an API.

## Configuration

Configuring the plugin is straightforward, you can add it on top of an
existing API by executing the following request on your Kong server:

```bash
$ curl -X POST http://kong:8001/apis/{api}/plugins \
    --data "name=canary" \
    --data "config.percentage=50"
```

`api`: The `id` or `name` of the API that this plugin configuration will target.

You can also apply it for every API using the `http://kong:8001/plugins/` endpoint.

| form parameter | default | description |
| --- | --- | --- |
| `name` | `canary` | The name of the plugin to use, in this case: `canary` |
| `config.start` |  | Future time in seconds since epoch, when the release will start (ignored when `percentage` is set, or when using `allow` or `deny`)|
| `config.duration` | `3600` | How long, in seconds, should the transition take (ignored when `percentage` is set, or when using `allow` or `deny`)|
| `config.percentage` |  | Fixed % of traffic to be routed to new target, if given overrides `start` and `duration` |
| `config.steps` | `1000` | Number of steps the release should be broken into |
| `config.upstream_host` |  | Target hostname where traffic will be routed, required if `upstream_uri/port` is not set  |
| `config.upstream_fallback` | false | Whether the plugin should fallback to the original upstream if the canary upstream doesn't have at least one healthy target. `upstream_host` **must** point to a valid Kong Upstream entity |
| `config.upstream_port` |  | Target port where traffic will be routed, required if `upstream_uri/host` is not set  |
| `config.upstream_uri` |  | Upstream URI where traffic will be routed, required if `upstream_host/port` is not set |
| `config.hash` | `consumer` | Entity to be used for hashing. Options: `consumer`, `ip`, `allow`, `deny`, or `none`. Please make sure when using `consumer` or `ip`, to properly set the settings for trusted ips (see settings `trusted_ips` and `real_ip_header` in the Kong config file)|
| `config.groups`   |  | An array (of strings) with the groupnames that are allowed or denied. |

## Usage

The plugin will route traffic to 2 different upstream services, referred to as
A and B. The location of service A will be defined by the `upstream_url`
property of the api the plugin is configured on. The location of service B is
defined by the `config.upstream_host`, `config.upstream_port`, and/or
`config.upstream_uri` as configured on the plugin.

There are 3 modes of operation:

1. Set a fixed percentage to be routed to destination B. See parameter
   `config.percentage`.
2. Define an allow or deny group with consumers to use/not use destination
   B. The consumer-group association can be configured using the ACL plugin.
3. Set a period over which (in lineair time) the traffic will be moved over
   from destination A to B. See parameters `config.start` and `config.duration`.

### Determining where to route a request.

(this part does not apply to restricting by allow / deny)

The plugin defines a number of "buckets" (`config.steps`). Each of those can
be routed to either A or B. For example: 100 steps, and `precentage` at 10%.
Then 100 buckets will be created, of which 10 will be routed to upstream B,
and 90 will remain at A.

Which requests end up in a specific bucket is determined by the `config.hash`
parameter. When set to `consumer` then it is made sure that each consumer will
consistently end up in the same bucket. The effect being that once a bucket a
consumer belongs to is switched to B, it will then always go to B, and a
consumer will not "flip-flop" between A and B.
Alternatively if it is set to `ip` then the same concept applies, but based on
the originating ip address.

The downside of `consumer` and `ip` is that if any specific consumer or ip
is responsible for a more than average part of the load, the migration is not
nicely distributed. Eg. with `percentage` set to 50%, then 50% of either the
consumers or ips are rerouted, but not necessarily 50% of the requests.

When set to `none` then the requests will be nicely distributed, each bucket
will get the same number of requests, but in this case a consumer or ip might
be flip-flopping between destination A and B on consecutive requests.

In any case there is an automatic fallback in case a consumer or ip could
not be identified for some reason. The fall-back order will be
`consumer`->`ip`->`none`.

### Finalizing the canary.

Once the canary is complete, either going to 100% for a percent-based canary,
or after the timed canary reached 100%, the configuration needs to be updated.

This takes 2 steps:

1. update location A to point to location B. This can be done by a `PATCH`
   request on the API where the `upstream_url` property is updated to the
   url as specified by `config.upstream_host`, `config.upstream_port` and/or
   `config.upstream_uri` (or location B).
2. Since now the location A and B are the same, the canary plugin can now be
   removed from the system with a `DELETE` request.

If the canary was not complete yet, then executing those steps prematurely,
will instantly switch 100% of traffic to the new location (B).

### Upstream Healthchecks

The configuration item `upstream_fallback` uses Kong's upstream healthchecker
to skip applying the canary upstream if it does not have at least one healthy
target. For this configuration to take effect, the following conditions must be
met:

- As this configuration relies on Kong's balancer (and healthchecks), the name
specified in `config.upstream_host` **must** point to a valid Kong
[Upstream object][upstream-object]
- Healthchecks are enabled in the canary upstream - i.e., the upstream specified
in `upstream_host` needs to have healthchecks enabled it. It works well with
both passive and active healthchecks. Check out the healthchecks documentation
[here][healthchecks-docs].

## Enterprise Support

Support, Demo, Training, API Certifications and Consulting
available at https://getkong.org/enterprise.


[upstream-object]: https://docs.konghq.com/0.14.x/admin-api/#upstream-objects
[healthchecks-docs]: https://docs.konghq.com/0.13.x/health-checks-circuit-breakers/
[badge-travis-url]: https://travis-ci.com/Kong/kong-plugin-enterprise-canary/branches
[badge-travis-image]: https://travis-ci.com/Kong/kong-plugin-enterprise-canary.svg?token=BfzyBZDa3icGPsKGmBHb&branch=master
