# kong-plugin-enterprise-degraphql

De-GraphQL is a plugin to create your own REST API on top of Kong by querying a
GraphQL service. Tecnology moves forward, but sometimes you still need to
support backwards.

## Usage

De-GraphQL needs a graphql endpoint to query. As an example, we are going to
build a REST API around https://swapi-graphql.eskerda.now.sh

Note this plugin differs from other plugins as far as configuration goes. It
needs to be activated on a service that points to a GraphQL endpoint
(sans `/graphql`).

### Start kong, and add the graphql endpoint as a service

```
KONG_PLUGINS=degraphql gojira up --volume $(realpath .):/kong-plugin
gojira shell
kong migrations bootstrap
kong start
```

```bash
http -f :8001/services name=example host=swapi-graphql.eskerda.now.sh protocol=https port=443
http -f :8001/services/example/routes hosts=example.com
# Test that it works
http :8000 Host:example.com query='{ allPeople { people { name } } }'
```

### Configure the plugin on the service

The plugin takes over the service. From this point on, the service represents
our REST api and not the graphql endpoint itself. Thus, it will return a 404 Not Found
if no degraphql routes have been configured.

```bash
http :8001/services/example/plugins name=degraphql
```

### Configure routes on the plugin

Once the plugin is activated on a service, we can add our own routes to build
our service, by defining uris and associating them to GraphQL queries.

```
http :8001/services/example/degraphql/routes uri='/people' query='{ allPeople { people { id name } } }'
http :8001/services/example/degraphql/routes uri='/planets' query='{ allPlanets { planets { id name climates terrains diameter } } }'
```

GET, POST and URL parameters are available through graphql variable scoping:

```
http :8001/services/example/degraphql/routes uri='/planets/:id' query='query one_planet ($id: ID) { planet(id: $id) { id name } }'
```

The same parameters are also allowed inside the graphql query as template variables:

```
http :8001/services/example/degraphql/routes uri='/people/:id' query='{ person (id: "{{id}}") { name {{fields}} } }'
```

```
http :8000/people Host:example.com
http :8000/planets Host:example.com
http :8000/planets/cGxhbmV0czo2MQ== Host:example.com
http :8000/people/cGVvcGxlOjE=?fields=gender,mass Host:example.com
```

### Available endpoints

#### GET :8001/services/example/degraphql/routes

List defined routes for service `example`

#### POST :8001/services/example/degraphql/routes

Create a route for service `example`. `uri` defines the path, and `query` the
GraphQL query associated with the `uri`.

#### PATCH :8001/services/example/degraphql/routes/some_id

Edit a route for service `example`, identified by `id`.

#### DELETE :8001/services/example/degraphql/routes/some_id

Delete a route for service `example`, identified by `id`.
