[![Build Status](https://travis-ci.com/Kong/kong-plugin-graphql-proxy-cache-advanced.svg?token=ucXdNBt8KcexACiALyaR&branch=master)](https://travis-ci.com/Kong/kong-plugin-graphql-proxy-cache-advanced)

# Proxy Caching GraphQL in Kong

## Configuration
Minimum viable GraphQL cache, work in progress.

### Enabling the plugin on a service
```
http :8001/services/{service}/plugins name=graphql-proxy-cache-advanced config.strategy=memory
```

### Parameters

* `config.cache_ttl`: Time-to-live, in seconds, of cache entities
* `config.strategy`: The backing data store in which to hold cache entities. Currently support `memory` only.
* `config.vary_headers`: Relevant headers considered for the cache key. If undefined, none of the headers are
taken into consideration

### Cache Key

Kong keys each cache elements based on the GraphQL query that is being send
in the HTTP request body. Internally, cache keys are represented as a
hexadecimal-encoded MD5 sum of the concatenation of the constituent parts.
This is calculated as follows:

```
key = md5( UUID | headers | body )
```

Where `headers` contains the headers defined in vary_headers. `vary_headers` defaults to NONE.

Kong will return the cache key associated with a given request as the
`X-Cache-Key` response header.

### Cache Status

Kong identifies the status of the request's proxy cache behavior via the `X-Cache-Status` header. There are several possible values for this header:

* `Miss`: The request could be satisfied in cache, but an entry for the resource was not found in cache, and the request was proxied upstream.
* `Hit`: The request was satisfied and served from cache.
* `Refresh`: The resource was found in cache, but could not satisfy the request, due to `Cache-Control` behaviors or reaching its hard-coded `cache_ttl` threshold.
* `Bypass`: The request could not be satisfied from cache based on plugin configuration.

## Admin API

This plugin provides several endpoints to managed cache entities. These endpoints are assigned to the `graphql-proxy-cache-advanced` resource.

The following endpoints are provided on the Admin API to examine and purge cache entities:

### Retrieve a Cache Entity

Two separate endpoints are available: one to look up a known plugin instance, and another that searches all graphql-proxy-cache-advanced plugins data stores for the given cache key. Both endpoints have the same return value.

#### Endpoint

`GET /graphql-proxy-cache-advanced/:plugin_id/caches/:cache_id`

| Attributes | Description |
| --- | --- |
| `plugin_id` | The UUID of the graphql-proxy-cache-advanced plugin |
| `cache_id` | The cache entity key as reported by the `X-Cache-Key` response header |

#### Endpoint

`GET /graphql-proxy-cache-advanced/:cache_id`

| Attributes | Description |
| --- | --- |
| `cache_id` | The cache entity key as reported by the `X-Cache-Key` response header |

#### Response

`HTTP 200 OK` if the cache entity exists; `HTTP 404 Not Found` if the entity with the given key does not exist.


### Delete Cache Entity

Two separate endpoints are available: one to look up a known plugin instance, and another that searches all graphql-proxy-cache-advanced plugins data stores for the given cache key. Both endpoints have the same return value.

#### Endpoint

`DELETE /graphql-proxy-cache-advanced/:plugin_id/caches/:cache_id`

| Attributes | Description |
| --- | --- |
| `plugin_id` | The UUID of the graphql-proxy-cache-advanced plugin |
| `cache_id` | The cache entity key as reported by the `X-Cache-Key` response header |

#### Endpoint

`DELETE /graphql-proxy-cache-advanced/:cache_id`

| Attributes | Description |
| --- | --- |
| `cache_id` | The cache entity key as reported by the `X-Cache-Key` response header |

#### Response

`HTTP 204 No Content` if the cache entity exists; `HTTP 404 Not Found` if the entity with the given key does not exist.


### Purge All Cache Entities

#### Endpoint

`DELETE /graphql-proxy-cache-advanced/`

#### Response

`HTTP 204 No Content` if the cache entity exists.

Note that this endpoint purges all cache entities across all `graphql-proxy-cache-advanced` plugins.
