[![Build Status][badge-travis-image]][badge-travis-url]

# kong-plugin-enterprise-rate-limiting

The Rate Limiting Advanced plugin for Kong Enterprise is a re-engineered version of the incredibly popular Kong Rate Limiting plugin, with greatly enhanced configuration options and performance.

## Configuration

### Enabling the plugin on a Service

#### With a database

Configure this plugin on a [Service](https://docs.konghq.com/latest/admin-api/#service-object) by making the following request:

```
$ curl -X POST http://kong:8001/services/{service}/plugins \
  --data name=rate-limiting-advanced \
  --data config.limit=10 \
  --data config.window_size=60 \
  --data config.sync_rate=10
```

#### Without a database

Configure this plugin on a [Service](https://docs.konghq.com/latest/admin-api/#service-object) by adding this section do your declarative configuration file:

```
plugins:
- name: rate-limiting-advanced
  service: {service}
  config:
```

In both cases, `{service}` is the `id` or `name` of the Service that this plugin configuration will target.


### Enabling the plugin on a Route

#### With a database

Configure this plugin on a [Route](https://docs.konghq.com/latest/admin-api/#Route-object) with:

```
$ curl -X POST http://kong:8001/routes/{route}/plugins \
  --data name=rate-limiting-advanced \
  --data config.limit=10 \
  --data config.window_size=60 \
  --data config.sync_rate=10
```

#### Without a database

Configure this plugin on a [Route](https://docs.konghq.com/latest/admin-api/#route-object) by adding this section do your declarative configuration file:

```
plugins:
- name: rate-limiting-advanced
  route: {route}
  config:
```

In both cases, `{route}` is the `id` or `name` of the Route that this plugin configuration will target.

### Enabling the plugin on a Consumer

#### With a database

You can use the `http://localhost:8001/plugins` endpoint to enable this plugin on specific [Consumers](https://docs.konghq.com/latest/admin-api/#Consumer-object):

```
$ curl -X POST http://kong:8001/consumers/{consumer}/plugins \
  --data name=rate-limiting-advanced \
  --data config.limit=10 \
  --data config.window_size=60 \
  --data config.sync_rate=10
```

#### Without a database

Configure this plugin on a [Consumer](https://docs.konghq.com/latest/admin-api/#Consumer-object) by adding this section do your declarative configuration file:

```
plugins:
- name: rate-limiting-advanced
  consumer: {consumer}
  config:
```

In both cases, `{consumer}` is the `id` or `username` of the Consumer that this plugin configuration will target.

You can combine `consumer_id` and `service_id`

In the same request, to furthermore narrow the scope of the plugin.

### Global plugins

- **Using a database**, all plugins can be configured using the `http://kong:8001/plugins/` endpoint.
- **Without a database**, all plugins can be configured via the `plugins:` entry on the declarative configuration file.

A plugin which is not associated to any Service, Route or Consumer (or API, if you are using an older version of Kong) is considered "global", and will be run on every request. Read the [Plugin Reference](https://docs.konghq.com/latest/admin-api/#add-plugin) and the [Plugin Precedence](https://docs.konghq.com/latest/admin-api/#precedence)sections for more information.

## Parameters

Here's a list of all the parameters which can be used in this plugin's configuration:

| Form Parameter | default | description
|----------------|---------|-------------
| `name`|| The name of the plugin to use, in this case: `rate-limiting-advanced`.
| `service_id`|| The id of the Service which this plugin will target.
| `route_id` || The id of the Route which this plugin will target.
| `enabled` | `true` | Whether this plugin will be applied.
| `consumer_id` || The id of the Consumer which this plugin will target.
|`config.limit`|| one of more request per window to apply
|`config.window_size`||One more more window sizes to apply (defined in seconds).
|`config.identifier` | `consumer` | How to define the rate limit key. Can be `ip`, `credential`, `consumer`, `service`, or `header`.
|`config.header_name` <br>*semi-optional* || Header name to use as the rate limit key when the `header` identifier is defined.
|`config.dictionary_name`| `kong_rate_limiting_counters` | The shared dictionary where counters will be stored until the next sync cycle.
|`config.sync_rate` | | How often to sync counter data to the central data store. A value of 0 results in synchronous behavior; a value of -1 ignores sync behavior entirely and only stores counters in node memory. A value greater than 0 will sync the counters in that many number of seconds.
|`config.namespace` <br>*optional* | `random string`|The rate limiting library namespace to use for this plugin instance. Counter data and sync configuration is shared in a namespace.
|`config.strategy`| `cluster` | The sync strategy to use; `cluster` and `redis` are supported.
|`config.redis.host` <br>*semi-optional* | | Host to use for Redis connection when the `redis` strategy is defined.
|`config.redis.port` <br>*semi-optional* ||Port to use for Redis connection when the `redis` strategy is defined.
|`config.redis.timeout` <br>*semi-optional* | | Connection timeout to use for Redis connection when the `redis` strategy is defined.
|`config.redis.password` <br>*semi-optional* || Password to use for Redis connection when the `redis` strategy is defined. If undefined, no AUTH commands are sent to Redis.
|`config.redis.database` <br>*semi-optional* |`0`|Database to use for Redis connection when the redis strategy is defined.
|`config.redis.sentinel_master` <br>*semi-optional* ||Sentinel master to use for Redis connection when the `redis` strategy is defined. Defining this value implies using Redis Sentinel.
|`config.redis.sentinel_role` <br>*semi-optional* ||Sentinel role to use for Redis connection when the `redis` strategy is defined. Defining this value implies using Redis Sentinel.
|`config.redis.sentinel_addresses` <br>*semi-optional* ||Sentinel addresses to use for Redis connection when the `redis` strategy is defined. Defining this value implies using Redis Sentinel.
|`config.redis.cluster_addresses` <br>*semi-optional* ||Cluster addresses to use for Redis connection when the redis strategy is defined. Defining this value implies using Redis cluster.
|`config.window_type`| `sliding` | This sets the time window to either `sliding` or `fixed`
|`config.retry_after_jitter_max` | `0` | The upper bound of a jitter (random delay) in seconds to be added to the `Retry-After` header of denied requests (status = 429) in order to prevent all the clients from coming back at the same time. The lower bound of the jitter is `0`; in this case, the `Retry-After` header is equal to the `RateLimit-Reset` header.

**Note:  Redis configuration values are ignored if the "cluster" strategy is used.**

**Note: PostgreSQL 9.5+ is required when using the "cluster" strategy with "postgres" as the backing Kong cluster data store. This requirement varies from the PostgreSQL 9.4+ requirement as described in the <a href="/install/source">Kong Community Edition documentation</a>.**

**Note: The `dictionary_name` directive was added to prevent the usage of the `kong` shared dictionary, which could lead to `no memory` errors**


## Notes
An arbitrary number of limits/window sizes can be applied per plugin instance. This allows users to create multiple rate limiting windows (e.g., rate limit per minute and per hour, and/or per any arbitrary window size); because of limitation with Kong's plugin configuration interface, each *nth* limit will apply to each *nth* window size. For example:

```
$ curl -i -X POST http://kong:8001/services/{service}/plugins \
  --data name=rate-limiting-advanced \
  --data config.limit=10,100 \
  --data config.window_size=60,3600 \
  --data config.sync_rate=10
```
This will apply rate limiting policies, one of which will trip when 10 hits have been counted in 60 seconds, or when 100 hits have been counted in 3600 seconds.

[badge-travis-url]: https://travis-ci.com/Kong/kong-plugin-enterprise-rate-limiting/branches
[badge-travis-image]: https://travis-ci.com/Kong/kong-plugin-enterprise-rate-limiting.svg?token=BfzyBZDa3icGPsKGmBHb&branch=master
