# kong-plugin-vault-auth

Authenticate with Vault. Do cool things. Save the planet.

## Status

Still a work in progress, but it can authenticate with a KV v1 Vault instance,
and if you ask nicely it will pour you a glass of orange juice.

## Usage

1. Start Vault and define a v1 KV mount:

```bash
$ vault server -dev

...

Unseal Key: jM/hpSdLMMsPVxBribFAeUSOYO5Ngk+LPOWPs0DW8xg=
Root Token: s.k8OWrubBGEigNUh32G73cCzc

$ vault secrets enable -path=kong-auth kv
Success! Enabled the kv secrets engine at: kong-auth/
```

2. Define a Vault instance:

```
vagrant@ubuntu-bionic:/kong$ http :8001/vaults name=dev host=127.0.0.1 token=s.k8OWrubBGEigNUh32G73cCzc mount=kong-auth
HTTP/1.1 201 Created
Access-Control-Allow-Origin: *
Connection: keep-alive
Content-Length: 212
Content-Type: application/json; charset=utf-8
Date: Wed, 23 Jan 2019 18:44:22 GMT
Server: kong/1.0.2

{
    "created_at": 1548269062,
    "host": "127.0.0.1",
    "id": "6df741db-0031-4ed2-a769-e35defd3b900",
    "mount": "kong-auth",
    "name": "dev",
    "port": 8200,
    "protocol": "http",
    "token": "s.k8OWrubBGEigNUh32G73cCzc",
    "updated_at": 1548269062
}
```

3. Create a Consumer:

```
vagrant@ubuntu-bionic:/kong$ http :8001/consumers/ username=bob
HTTP/1.1 201 Created
Access-Control-Allow-Origin: *
Connection: keep-alive
Content-Length: 103
Content-Type: application/json; charset=utf-8
Date: Wed, 23 Jan 2019 18:45:48 GMT
Server: kong/1.0.2

{
    "created_at": 1548269148,
    "custom_id": null,
    "id": "d45b52b0-51e1-48ab-bbc9-18d4911f01d8",
    "username": "bob"
}
```

4. Create a Vault credential for the Consumer:

```
vagrant@ubuntu-bionic:/kong$ http POST :8001/vaults/dev/credentials/bob
HTTP/1.1 201 Created
Access-Control-Allow-Origin: *
Connection: keep-alive
Content-Length: 191
Content-Type: application/json; charset=utf-8
Date: Wed, 23 Jan 2019 18:46:41 GMT
Server: kong/1.0.2

{
    "data": {
        "access_token": "szczpbTLyjDPIoU2XwYA3Xi8tl9BLoRH",
        "consumer": {
            "id": "d45b52b0-51e1-48ab-bbc9-18d4911f01d8"
        },
        "created_at": 1548269201,
        "secret_token": "hCStAPeeYP85trlBqo7g8dLmCxnJdEw1"
    }
}
```

Note that `access_token` and `secret_token` are auto-generated, but can be specified.

5. Create a Vault plugin on the appropriate target

```
vagrant@ubuntu-bionic:/kong$ http :8001/services/test/plugins name=vault-auth config.vault.id=6df741db-0031-4ed2-a769-e35defd3b900 -f
HTTP/1.1 201 Created
Access-Control-Allow-Origin: *
Connection: keep-alive
Content-Length: 430
Content-Type: application/json; charset=utf-8
Date: Wed, 23 Jan 2019 18:48:16 GMT
Server: kong/1.0.2

{
    "config": {
        "access_token_name": "access_token",
        "anonymous": null,
        "hide_credentials": false,
        "run_on_preflight": true,
        "secret_token_name": "secret_token",
        "tokens_in_body": false,
        "vault": {
            "id": "6df741db-0031-4ed2-a769-e35defd3b900"
        }
    },
    "consumer": null,
    "created_at": 1548269296,
    "enabled": true,
    "id": "00ddc0b8-fa15-41b9-b075-d36db9f127d0",
    "name": "vault-auth",
    "route": null,
    "run_on": "first",
    "service": {
        "id": "c9f56d57-f8d2-4797-8432-c98ad8e05bbe"
    }
}
```

7. Authenticate requests for glory and honor

```
vagrant@ubuntu-bionic:/kong$ http :8000/get access_token:szczpbTLyjDPIoU2XwYA3Xi8tl9BLoRH secret_token:hCStAPeeYP85trlBqo7g8dLmCxnJdEw1
HTTP/1.1 200 OK
Access-Control-Allow-Credentials: true
Access-Control-Allow-Origin: *
Connection: keep-alive
Content-Length: 512
Content-Type: application/json
Date: Wed, 23 Jan 2019 18:48:55 GMT
Server: gunicorn/19.9.0
Via: kong/1.0.2
X-Kong-Proxy-Latency: 180
X-Kong-Upstream-Latency: 145

{
    "args": {},
    "headers": {
        "Accept": "*/*",
        "Accept-Encoding": "gzip, deflate",
        "Access-Token": "szczpbTLyjDPIoU2XwYA3Xi8tl9BLoRH",
        "Connection": "close",
        "Host": "httpbin.org",
        "Secret-Token": "hCStAPeeYP85trlBqo7g8dLmCxnJdEw1",
        "User-Agent": "HTTPie/0.9.8",
        "X-Consumer-Id": "d45b52b0-51e1-48ab-bbc9-18d4911f01d8",
        "X-Consumer-Username": "bob",
        "X-Forwarded-Host": "localhost"
    },
    "origin": "127.0.0.1, 98.173.7.34",
    "url": "http://localhost/get"
}
```

## Tests

busted specs are included in this repo. It's expected that a Vault server is
present and running in dev mode via `vault server -dev`, and that a v1 KV mount
at `kong-auth` is enabled.
