-- This software is copyright Kong Inc. and its licensors.
-- Use of the software is subject to the agreement between your organization
-- and Kong Inc. If there is no such agreement, use is governed by and
-- subject to the terms of the Kong Master Software License Agreement found
-- at https://konghq.com/enterprisesoftwarelicense/.
-- [ END OF LICENSE 0867164ffc95e54f04670b5169c09574bdbd9bba ]

local Node = require'kong.gql.query.nodes.node'
local FragmentSpread = require'kong.gql.query.nodes.fragment_spread'
local FragmentDefNode = require'kong.gql.query.nodes.fragment_def_node'
local OperationNode = require"kong.gql.query.nodes.operation_node"
local util = require'kong.gql.util'

local _DocumentNode = Node:extend()


-- Entry point to the AST
-- @param definitions: [OperationNode|FragmentDefNode]
-- @param schema: table, a Schema object
function _DocumentNode:new(definitions)
    _DocumentNode.super.new(self)
    self.definitions = definitions
    self.frag_def_map = util.lst_to_map(
            definitions,
            function (def) return def.name end,
            function (def) return def:is(FragmentDefNode) end
    )
end


function _DocumentNode:children()
    return util.lst_to_iter(self.definitions)
end


-- Returns FragmentDefNode object corresponding name
-- @param name: String
function _DocumentNode:get_frag_def(name)
    return self.frag_def_map[name]
end


function _DocumentNode:get_op_node()
    for def in util.lst_to_iter(self.definitions) do
        if def:is(OperationNode) then return def end
    end

    return error("GraphQL query has no root operation")
end


function _DocumentNode:__tostring() return 'DocumentNode' end


return _DocumentNode
