-- This software is copyright Kong Inc. and its licensors.
-- Use of the software is subject to the agreement between your organization
-- and Kong Inc. If there is no such agreement, use is governed by and
-- subject to the terms of the Kong Master Software License Agreement found
-- at https://konghq.com/enterprisesoftwarelicense/.
-- [ END OF LICENSE 0867164ffc95e54f04670b5169c09574bdbd9bba ]

local util = require'kong.gql.util'
local Node = require'kong.gql.query.nodes.node'

local _SelectableNode = Node:extend()


-- Not to be confused with the non-terminal Selection
-- This actually represents nestable nodes with attribute `selection_set`
function _SelectableNode:new(name, selection_set, directives)
    _SelectableNode.super.new(self)
    self.name = name
    self.selection_set = selection_set or {}
    self.directives = directives or {}

    self.selection_map = util.lst_to_map( -- for debugging. Only field nodes here.
            self.selection_set,
            function (node)
                if node.alias then return node.alias
                else return node.name
                end
            end,
            function (node)
                return node:is(require'kong.gql.query.nodes.field_node')
            end
    )
end


function _SelectableNode:children()
    return util.lst_to_iter(self.selection_set)
end


function _SelectableNode:select(key)
    return self.selection_map[key]
end


return _SelectableNode
