-- This software is copyright Kong Inc. and its licensors.
-- Use of the software is subject to the agreement between your organization
-- and Kong Inc. If there is no such agreement, use is governed by and
-- subject to the terms of the Kong Master Software License Agreement found
-- at https://konghq.com/enterprisesoftwarelicense/.
-- [ END OF LICENSE 0867164ffc95e54f04670b5169c09574bdbd9bba ]

-- Type Decorator Language

local gmatch = string.gmatch

local GQL_NAME_PATTERN = "[_A-Za-z][_A-za-z0-9]*"


local _Path = {}


function _Path.new()
    local self = {}
    self.context = {}
    _Path.__index = _Path
    return setmetatable(self, _Path)
end


function _Path:touch(tok)
    if not tok then error("Invalid path token.") end

    local res = self.context[tok]
    if not res then
        local n = _Path.new()
        self.context[tok] = n
        return n
    end

    return res
end


function _Path:get(tok)
    return self.context[tok]
end


function _Path:terminate()
    self.terminated = true
end


local _Tdn = {}


function _Tdn.new(types)
    local self = {}

    self.queue = {}
    self.path = _Path.new()
    for _, type_stmt in ipairs(types) do
        local path = self.path
        local it = gmatch(type_stmt, GQL_NAME_PATTERN)
        local next = it()
        while next ~= nil do
            path = path:touch(next)
            next = it()
        end
        path:terminate()
    end
    _Tdn.__index = _Tdn
    return setmetatable(self, _Tdn)
end


-- @deprecated Must keep track of current paths as well as next type
function _Tdn:next_token(type_name, tok)
    local new_queue = {}
    if tok then
        for _, path in ipairs(self.queue) do
            local candidate = path:get(tok)
            if candidate ~= nil then table.insert(new_queue, candidate) end
        end -- Continue candidate queue
    end

    local type_path = self.path:get(type_name) -- Add new candidates to queue if type triggered
    if type_path ~= nil then table.insert(new_queue, type_path) end

    local o = {}
    o.queue = new_queue
    self.__index = self
    return setmetatable(o, self) -- TODO: Might clog garbage
end


function _Tdn:should_decorate()
    for _, path in ipairs(self.queue) do
        if path.terminated then
            return true
        end
    end
    return false
end


return _Tdn
