-- This software is copyright Kong Inc. and its licensors.
-- Use of the software is subject to the agreement between your organization
-- and Kong Inc. If there is no such agreement, use is governed by and
-- subject to the terms of the Kong Master Software License Agreement found
-- at https://konghq.com/enterprisesoftwarelicense/.
-- [ END OF LICENSE 0867164ffc95e54f04670b5169c09574bdbd9bba ]


-- Converts list to set indexed by key function `f`
-- @param lst: table
-- @param f function
-- @param(opt) filter: function
local function lst_to_map(lst, f, filter)
    local s = {}
    for _, elem in ipairs(lst) do
        if not filter or filter(elem) then
            s[f(elem)] = elem
        end
    end
    return s
end


local function map_lst (lst, f)
    local r = {}
    for _, elem in ipairs(lst) do
        table.insert(r, f(elem))
    end
    return r
end


-- Turn list to iterator
-- @param lst: table
-- @param(opt) filter: function, skips element if not truthy
local function lst_to_iter(lst, filter)
    local i = 0
    local function iter()
        i = i + 1
        if i > #lst then return nil
        elseif filter and not filter(lst[i]) then return iter() end

        return lst[i]
    end
    return iter
end


local function filter_lst(lst, filter)
    local r = {}
    for _, elem in ipairs(lst) do
        if filter(elem) then table.insert(r, elem) end
    end
    return r
end


-- Incorporates `m` into `o` by mutating `o`.
-- @return nil if collision exists, true otherwise
local function map_consume(o, m)
    for k, v in pairs(m) do
        if o[k] then return nil end
        o[k] = v
    end
    return true
end


-- Return an iterator filtered by boolean function `filter`
-- @param iter: iterator
-- @param filter: function, skips element if not truthy
local function filter_iter(iter, filter)
    local r
    local function iter_f()
        r = iter()
        if not r then return nil
        elseif not filter(r) then return iter_f() end

        return r
    end
    return iter_f
end


return {
    lst_to_map = lst_to_map,
    lst_to_iter = lst_to_iter,
    filter_iter = filter_iter,
    filter_lst = filter_lst,
    map_consume = map_consume,
    map_lst = map_lst
}
